<?php

namespace Test\Test\Policies;

use App\User;
use Test\Test\Models\Test;

class TestPolicy
{

    /**
     * Determine if the given user can view the test.
     *
     * @param User $user
     * @param Test $test
     *
     * @return bool
     */
    public function view(User $user, Test $test)
    {
        if ($user->canDo('test.test.view') && $user->is('admin')) {
            return true;
        }

        if ($user->canDo('blocks.block.view') 
        && $user->is('manager')
        && $block->user->parent_id == $user->id) {
            return true;
        }

        return $user->id === $test->user_id;
    }

    /**
     * Determine if the given user can create a test.
     *
     * @param User $user
     * @param Test $test
     *
     * @return bool
     */
    public function create(User $user)
    {
        return  $user->canDo('test.test.create');
    }

    /**
     * Determine if the given user can update the given test.
     *
     * @param User $user
     * @param Test $test
     *
     * @return bool
     */
    public function update(User $user, Test $test)
    {
        if ($user->canDo('test.test.update') && $user->is('admin')) {
            return true;
        }

        if ($user->canDo('blocks.block.update') 
        && $user->is('manager')
        && $block->user->parent_id == $user->id) {
            return true;
        }

        return $user->id === $test->user_id;
    }

    /**
     * Determine if the given user can delete the given test.
     *
     * @param User $user
     * @param Test $test
     *
     * @return bool
     */
    public function destroy(User $user, Test $test)
    {
        if ($user->canDo('test.test.delete') && $user->is('admin')) {
            return true;
        }

        if ($user->canDo('blocks.block.delete') 
        && $user->is('manager')
        && $block->user->parent_id == $user->id) {
            return true;
        }

        return $user->id === $test->user_id;
    }

    /**
     * Determine if the given user can verify the given test.
     *
     * @param User $user
     * @param Test $test
     *
     * @return bool
     */
    public function verify(User $user, Test $test)
    {
        if ($user->canDo('test.test.verify') && $user->is('admin')) {
            return true;
        }

        if ($user->canDo('test.test.verify') 
        && $user->is('manager')
        && $test->user->parent_id == $user->id) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given test.
     *
     * @param User $user
     * @param Test $test
     *
     * @return bool
     */
    public function approve(User $user, Test $test)
    {
        if ($user->canDo('test.test.approve') && $user->is('admin')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperUser()) {
            return true;
        }
    }
}
